from pychemkin import ReactionCoeffs


def test_str():
    """
    testing override of __str__ for ReactionCoeffs class
    """
    rc = ReactionCoeffs('Constant', k = 1e3)
    correct = "Constant Reaction Coeffs:"
    assert str(rc)[:len(correct)] == correct

def test_repr():
    """
    testing override of __repr__ for ReactionCoeffs class
    """
    rc = ReactionCoeffs('modifiedArrhenius', A = 1e7, E=1e3, T=1e2, b=0.5)
    rc1 = eval(repr(rc))
    assert rc == rc1

def test_insufficient():
    """
    test when reaction coefficient params are not suitable for reaction type
    """
    rc = ReactionCoeffs('modifiedArrhenius', A = 1e7, E=1e3, b=0.5)
    try:
        rc.k_forward()
    except ValueError as e:
        assert type(e) == ValueError
    rc = ReactionCoeffs('Arrhenius', b=0.5)
    try:
        rc.k_forward()
    except ValueError as e:
        assert type(e) == ValueError
    rc = ReactionCoeffs('Constant', A = 1e7, E=1e3, b=0.5)
    try:
        rc.k_forward()
    except ValueError as e:
        assert type(e) == ValueError
        print(e)

def test_bad_arg():
    """
    test for incorrect or extraneous arguments
    """
    rc = ReactionCoeffs('modifiedArrhenius', A = 1e7, E=1e3, b=(0.5,8j), T=1000)
    try:
        rc.k_forward()
    except TypeError as e:
        assert type(e) == TypeError
        print(e)
    try:
        rc.set_params(b=0.5,T=-1)
        rc.k_forward()
    except ValueError as e:
        assert type(e) == ValueError
        print(e)
    try:
        rc.set_params(b=0.5,T=1,A=-1)
        rc.k_forward()
    except ValueError as e:
        assert type(e) == ValueError
        print(e)
    rc = ReactionCoeffs('Arrhenius', A = 1e7, E=1e3, T=-1000)
    try:
        rc.k_forward()
    except ValueError as e:
        assert type(e) == ValueError
        print(e)
    try:
        rc.set_params(A=-1,T=1)
        rc.k_forward()
    except ValueError as e:
        assert type(e) == ValueError
        print(e)

def test_not_implement():
    """test of hooks (things not yet implemented)"""
    try:
        rc = ReactionCoeffs('HelloWorld', A = 1e7, E=1e3, b=0.5)
    except NotImplementedError as e:
        assert type(e) == NotImplementedError
        print(e)



from os import path
import numpy as np
from .SQLParser import SQLParser
from .InputParser import InputParser
from .ReactionCoeffs import ReactionCoeffs
from .BackwardCoeffs import BackwardCoeffs


class chemkin:

    '''
    This chemkin module computes the the reaction rates/ progress rates of the species
    given the temperature and species concentrations

    METHODS and ATTRIBUTES
    ========
    After initialization, user could call:
     - reaction_rate(x, T): method to calculate reaction rate given concentration x and temperature T.
     - species: A sequence (list, array, tuple) containing the names of the species
     - progress_rate(x): calculate progress rate given x (assumes temperature is already set)
     - _set_rc_params(T=..., R=..., A=...): internal method to set params of reaction coeffs

    EXAMPLES
    =========
    >>> chem = chemkin("tests/test_xml/rxns.xml")
    Finished reading xml input file
    >>> print(chem.species)
    ['H', 'O', 'OH', 'H2', 'H2O', 'O2']
    >>> chem.reaction_rate([1,1,1,1,1,1], 1000)
    array([ -6.28889929e+06,   6.28989929e+06,   6.82761528e+06,
            -2.70357993e+05,   1.00000000e+03,  -6.55925729e+06])
    '''

    def __init__(self, file_name):
        '''
        INPUT
        =====
        file_name: string, required
                   xml file containing species and chemical reaction information
        '''
        self.file_name = file_name

        input_ = InputParser(file_name)

        self.rxndata = input_.reactions
        rc_list = [ReactionCoeffs(**params) for params in input_.rate_coeff_params]
        equationlist = []
        rxn_types = []
        reversible = []

        for i, reaction in enumerate(self.rxndata):
            equationlist.append(reaction['equation'])
            rxn_types.append(reaction['type'])
            reversible.append(reaction['reversible'].strip().lower())

        reversible = np.array(reversible)

        # check whether `reversible` is valid
        if not np.all(np.logical_or(reversible == 'yes', reversible == 'no')):
            raise ValueError('`reversible` should be either "yes" or "no".')

        reversible = reversible == 'yes'
        if np.any(reversible):
            species_reversible = np.sum(input_.nu_react[:, reversible] + input_.nu_prod[:, reversible], axis=1) > 0
        else:
            species_reversible = np.array([False for _ in range(len(input_.species))])

        if np.any(species_reversible):
            the_shape = (np.sum(species_reversible),np.sum(reversible))
            backward_coeffs = BackwardCoeffs(input_.nu_react[:, reversible][species_reversible,:].reshape(the_shape), \
                                             input_.nu_prod[:, reversible][species_reversible,:].reshape(the_shape), \
                                             np.array(input_.species)[species_reversible])
        else:
            backward_coeffs = BackwardCoeffs(np.array([]), np.array([]), np.array([]))

        self.nu_react = input_.nu_react
        self.nu_prod = input_.nu_prod

        if self.nu_prod.shape != self.nu_react.shape or len(rc_list) != self.nu_prod.shape[1]:
            raise ValueError("Dimensions not consistent!")
        if np.any(self.nu_react<0.0) or np.any(self.nu_prod<0.0):
            raise ValueError("Negative stoichiometric coefficients are prohibited!")

        self.rc_list = rc_list
        self.bc = backward_coeffs
        self.species = input_.species
        self.equations = equationlist
        self.rxn_types = rxn_types

        self.reversible = reversible
        self.species_reversible = species_reversible
        if np.any(np.array(self.rxn_types)!='Elementary'):
            raise NotImplementedError('Only elementary reactions accepted')
        self.T = None

    def _set_rc_params(self,**kwargs):
        ''' add new or change old parameters for all reaction coeffs.
        '''
        if 'T' in kwargs:
            self.T = kwargs['T']
        for rc in self.rc_list:
            rc.set_params(**kwargs)

    def __repr__(self):
        class_name = type(self).__name__
        args = "'{}'".format(self.file_name)
        return class_name + "(" + args +")"

    def __len__(self):
        '''return number of reactions'''
        return len(self.rc_list)

    def __str__(self):
        print(self.equations)
        eqn_str = "chemical equations:\n[\n" + "\n".join([str(eq_) for eq_ in self.equations]) + "\n]"
        species_str = "species: " + str(self.species)
        species_str = "species: not specified"
        nu_react_str = "nu_react:\n" + str(self.nu_react)
        nu_prod_str = "nu_prod:\n" + str(self.nu_prod)
        rc_str = "reaction coefficients:\n[\n" + "\n".join([str(rc_) for rc_ in self.rc_list]) + "\n]"
        rxn_str = "reaction types: " + str(self.rxn_types)
        reversible_str = "reversible: " + str(self.reversible)
        return "\n".join([eqn_str, species_str,nu_react_str,nu_prod_str,rc_str, rxn_str, reversible_str])

    def progress_rate(self, x, T):
        '''Return progress rate for a system of M reactions involving N species

        INPUTS
        =======
        x: array or list, required
           A length-N vector specifying concentrations of the N species
        T: float, optional
           Temperature in K
        RETURNS
        ========
        Length-N array of reaction rates of species
        '''
        self._set_rc_params(T=T)
        x = np.array(x).reshape(-1,1)
        if len(x) != self.nu_prod.shape[0]:
            raise ValueError("ERROR: The concentration vector x must be of length N, where N is the \
            number of species")
        #check that concentrations are all non-negative:
        if np.any(x<0):
            raise ValueError("ERROR: All the species concentrations must be non-negative")

        #initialize progress rate vector
        kf = np.array([rc.k_forward() for rc in self.rc_list])
        pr = kf * np.product(x ** self.nu_react, axis=0)

        if np.any(self.reversible):
            pr[self.reversible] = pr[self.reversible] - self.bc.backward_coeffs(kf[self.reversible], self.T) \
            * np.product(x ** self.nu_prod[:, self.reversible], axis=0)
        return pr

    def reaction_rate(self, x, T):
        '''
        Return reaction rates for a system of M reactions involving N species
        INPUTS
        =======
        x: array or list, required
           A length-N vector specifying concentration of each specie
        T: float, optional
           Temperature in K
        RETURNS
        ========
        R: Array of reaction rates of species (length N)
        '''
        self._set_rc_params(T=T)
        r = self.progress_rate(x, T)

        # Return an array...
        return np.sum(r * (self.nu_prod-self.nu_react), axis=1)



import numpy as np

class ReactionCoeffs:
    """
    A class for computing forward reaction coefficients

    set_params: sets reaction rate coefficient parameters
    k_forward: compute forward reaction rate coefficient

    EXAMPLES
    =========
    >>> rc = ReactionCoeffs('Constant', k = 1e3)
    >>> rc.k_forward()
    1000.0
    >>> rc = ReactionCoeffs('Arrhenius', A = 1e7, E=1e3)
    >>> rc.set_params(T=1e2)
    >>> rc.k_forward()
    3003549.0889639612
    """

    def __init__(self, type, **kwargs):
        '''
        INPUT
        =====
        type: string, required
              Type of the reaction rate coefficient of interest (e.g. "Constant", "Arrhenius", etc)
        '''
        self.__rtype = str(type)
        self.__params = kwargs
        if 'R' not in self.__params:
            self.__params['R'] = 8.314

    def set_params(self, **kwargs):
        for param in kwargs:
            self.__params[param] = kwargs[param]

    def __str__(self):
        return self.__rtype + " Reaction Coeffs: " + str(self.__params)

    def __repr__(self):
        class_name = type(self).__name__
        params_str = ""
        for param in self.__params:
            params_str += " ,"+param+" = "+str(self.__params[param])
        return str(class_name)+'("'+self.__rtype+'"'+params_str+')'

    def __eq__(self,other):
        '''
        Check if two forward reaction rate coefficients are the same. They are same if the k values are equal.
        '''
        return self.k_forward() == other.k_forward()

    def __check_param_in(self, param_list):
        '''
        Returns dictionary of parameter values
        INPUT
        =====
        param_list: iterable, required
                    list of parameter namesof interest
        RETURNS
        =======
        Dictionary of parameters in provided list (returns None if mismatch present between parameter list and instance attributes)
        '''
        param_dict = dict()
        for param in param_list:
            if param in self.__params:
                param_dict[param] = self.__params[param]
            else:
                return None
        return param_dict

    def k_forward(self):
        '''
        Computes forward reaction rate coefficient

        NOTES
        ==========
        call corresponding private method to calculate k_values.

        RETURNS
        ==========
        k: (real number) Forward reaction rate coefficient
        '''
        if self.__rtype == "Constant":
            params = self.__check_param_in(['k'])
            if params != None:
                return self.__const(**params)
            else:
                raise ValueError('Insufficient Parameters')
        elif self.__rtype == "Arrhenius":
            params = self.__check_param_in(['A','E','T','R'])
            if params != None:
                return self.__arr(**params)
            else:
                raise ValueError('Insufficient Parameters')
        elif self.__rtype == "modifiedArrhenius":
            params = self.__check_param_in(['A','b','E','T','R'])
            if params != None:
                return self.__mod_arr(**params)
            else:
                raise ValueError('Insufficient Parameters')
        else:
            raise NotImplementedError("Type not supported yet!")

    def __const(self,k):
        '''
        Return constant coefficient
        INPUTS
        =======
        k: float, required
           constant reaction rate coefficient
        RETURNS
        ========
        k: float
           constant reaction rate coefficient
        '''

        if k < 0:
            raise ValueError("k cannot be negative!")
        return k

    def __arr(self,A,E,T,R):
        '''
        Return Arrhenius reaction rate coefficient
        INPUTS
        =======
        A: float, required
           Arrhenius prefactor  A. A  is strictly positive
        E: float, required
           Activation energy
        T: float, required
           Temperature. T must be positive (assuming a Kelvin scale)
        R: float, required
           Ideal gas constant
        RETURNS
        ========
        k: float
           Arrhenius reaction rate coefficient
        NOTES
        ========
        R = 8.314 is the default ideal gas constant.
        It should be positive (except to convert units)
        '''

        if A < 0.0:
            raise ValueError("A = {0:18.16e}:  Negative Arrhenius prefactor is prohibited!".format(A))

        if T < 0.0:
            raise ValueError("T = {0:18.16e}:  Negative temperatures are prohibited!".format(T))

        if R < 0.0:
            raise ValueError("R = {0:18.16e}:  Negative ideal gas constant is prohibited!".format(R))

        return A * np.exp(-E / (R * T))

    def __mod_arr(self,A,b,E,T,R):
        '''
        Return modified Arrhenius reaction rate coefficient
        INPUTS
        =======
        A: float, required
           Arrhenius prefactor  A. A  is strictly positive
        b: float, required
           Modified Arrhenius parameter b. b must be real
        E: float, required
           Activation energy
        T: float, required
           Temperature. T must be positive (assuming a Kelvin scale)
        R: float, required
           The ideal gas constant
        RETURNS
        ========
        k: Modified Arrhenius reaction rate coefficient
        NOTES
        ========
        R=8.314 is the default ideal gas constant
        '''
        if A < 0.0:
            raise ValueError("A = {0:18.16e}:  Negative Arrhenius prefactor is prohibited!".format(A))

        if T < 0.0:
            raise ValueError("T = {0:18.16e}:  Negative temperatures are prohibited!".format(T))

        if R < 0.0:
            raise ValueError("R = {0:18.16e}:  Negative ideal gas constant is prohibited!".format(R))

        return A * (T**b) * np.exp(-E / (R * T))



#change this once we rearrange directory structure
from pychemkin import chemkin
import numpy as np

def test_not_implement():
    """Test that input parser correctly handles reaction types and rate coefficients that are not implemented"""
    try:
        reactions = chemkin('tests/test_xml/rxns_bad_not_implement.xml')
    except NotImplementedError as e:
        assert type(e) == NotImplementedError
        print(e)

def test_bad_reactant():
    """test that input parser correctly handles reactants not matching the species array"""
    try:
        reactions = chemkin('tests/test_xml/rxns_bad_reactants.xml')
    except ValueError as e:
        assert type(e) == ValueError
        print(e)

def test_from_xml():
    """
    testing normal reaction rate calculations from xml input
    """
    reactions = chemkin('tests/test_xml/rxns.xml')
    expect = np.array([ -6.28889929e+06,   6.28989929e+06,   6.82761528e+06,
        -2.70357993e+05,   1.00000000e+03,  -6.55925729e+06])
    assert all(reactions.reaction_rate([1, 1, 1, 1, 1, 1], 1000).astype(int) == expect.astype(int))
    return reactions


def test_singlerxnxml():

    """testing that single reaction systems are handled correctly"""

    reactions = chemkin('tests/test_xml/rxns_single.xml')
    expect = np.array([ -1.e4, -1.e4, 1.e4, 1.e4])
    assert all(reactions.reaction_rate([1,1,1,1],1000).astype(int) == expect.astype(int))
    return reactions


def test_reaction_rate():
    """
    test reaction rate claculation
    """
    reactions = test_from_xml()
    expect = np.array([ -6.28889929e+06,   6.28989929e+06,   6.82761528e+06,
        -2.70357993e+05,   1.00000000e+03,  -6.55925729e+06])
    assert all(reactions.reaction_rate([1, 1, 1, 1, 1, 1],1000).astype(int) == expect.astype(int))

def test_construct_nonelementary():
    """
    test that chemkin object handles non-elementary reactions as expected
    """

    try:
        reactions = chemkin('tests/test_xml/rxns_pseudononelementary.xml')
        reactions.reaction_rate_T(np.ones(1), 1000)
    except NotImplementedError as e:
        assert type(e)==NotImplementedError
        print(e)

def test_invalid_reversible():
    """
    test that chemkin object handles invalid reversible input as expected
    """

    try:
        reactions = chemkin('tests/test_xml/rxns_invalid_reversible.xml')
    except ValueError as e:
        assert type(e) == ValueError
        print(e)

def test_repr():
    """
    test override of __repr__ function
    """
    reactions = test_from_xml()
    assert repr(reactions)=="chemkin('tests/test_xml/rxns.xml')"

def test_len():
    """
    test __len__ function
    """
    reactions = test_from_xml()
    assert len(reactions)==3

def test_str():
    """
    test __str__ function
    """
    reactions = test_from_xml()
    print(str(reactions))

def test_dimension_error():
    """
    check that inconsistencies in input params are handled correctly
    """
    try:
        reactions = chemkin('tests/test_xml/rxns_dimensionerror.xml')
    except ValueError as e:
        assert type(e) == ValueError
        print(e)

def test_bad_x():
    """
    test that errors with the shape of the concentration vector are handled correctly
    """
    reactions = test_from_xml()
    try:
        reactions.progress_rate([[1],[1]], 1000)
    except ValueError as e:
        assert type(e) == ValueError
        print(e)
    try:
        reactions.reaction_rate([[1],[1]], 10000)
    except ValueError as e:
        assert type(e) == ValueError
        print(e)
    try:
        reactions.reaction_rate([[1],[-1],[1],[1],[1],[1]], 1000)
    except ValueError as e:
        assert type(e) == ValueError
        print(e)

def test_long():
    reactions = chemkin('tests/test_xml/rxnset_long.xml')
    x = np.array([2., 1., 0.5, 1., 1., 0., 0., 0.25])
    T = 1500
    print(reactions.reaction_rate(x, T))
    expect = np.array([ -7.10942556e+12,  -1.58164670e+13,   2.20952666e+13,
        -1.65522031e+12,   1.12504921e+13,   0.00000000e+00,
         1.66470929e+13,  -2.54117388e+13])
    assert str(reactions.reaction_rate(x, T)) == str(expect)

def test_reversible():
    reactions = chemkin('tests/test_xml/rxns_reversible.xml')
    x = np.array([2., 1., 0.5, 1., 1., 0., 0., 0.25])
    T = 1500
    expect = np.array([  3.77689350e+14,  -3.81148508e+14,  -4.18872924e+14,   6.96275658e+12,
   3.07172197e+13,   3.93416746e+14,   1.66470991e+13,  -2.54117388e+13])
    assert str(reactions.reaction_rate(x, T)) == str(expect)

def test_singlereversible():
    reactions = chemkin('tests/test_xml/rxns_singlereversible.xml')
    x = np.array([2., 1., 0.5, 1.])
    T = 1500
    expect = np.array([  2.97178272e+14,   2.97178272e+14,  -2.97178272e+14,  -2.97178272e+14])
    assert str(reactions.reaction_rate(x, T)) == str(expect)

def test_mixedreversible():
    reactions = chemkin('tests/test_xml/rxnset_mixedreversible_long.xml')
    x = np.array([2., 1., 0.5, 1., 1., 0., 0., 0.25])
    T = 1500
    expect = np.array([ -1.51420660e+13,   1.22682375e+13,  -2.60415020e+13,   6.37742014e+12,
   3.13025562e+13,   0.00000000e+00,  1.66470929e+13,  -2.54117388e+13])
    assert str(reactions.reaction_rate(x, T)) == str(expect)



from pychemkin import SQLParser

def test_databasexistence():
    '''
    Test whether program correctly catches that database doesn't exist (or has incorrect path)
    '''
    try:
        parser = SQLParser()
    except ValueError as e:
        assert type(e) == ValueError
        print(e)

def test_missingspecie():
    '''
    Test whether SQL parser correctly handles a requested specie missing from the database
    '''
    try:
        parser = SQLParser()
        parser._get_coeffs('CH3CN', 700)
    except ValueError as e:
        assert type(e) == ValueError
        print(e)

def test_outofTrange():
    '''
    Test whether SQL parser correctly handles a temperature request that's out of the valid range
    '''
    try:
        parser = SQLParser()
        parser._get_coeffs('O2',4000)
    except ValueError as e:
        assert type(e) == ValueError
        print(e)

def test_get_species():
    '''
    Test get_species function
    '''
    parser = SQLParser()
    print(parser.get_species(1000))


def test_sql2pandas():
    '''
    Test sql2pandas function
    '''
    parser = SQLParser()
    print(parser.sql2pandas())



import xml.etree.ElementTree as ET
from copy import deepcopy
import numpy as np


class InputParser:
    '''
    The Input Parser class processes xml input used for subsequent chemical kinetics calculations

    Attributes
    ========
    'equation' - an equation of the chemical reaction to solve
    'id'- the number of reactions in the file
    'products'- the output of the chemical equation
    'rateCoeffParams'- the variables needed to calculate the reaction rate coefficient
    'reactants' - chemical reactants
    'reversible'- yes/no if a reversible reaction
    'type'- the type of reaction (i.e. 'elementary')
    EXAMPLES
    =========
    >>> input_ = InputParser('tests/test_xml/rxns.xml')
    Finished reading xml input file
    >>> print(input_.species)
    ['H', 'O', 'OH', 'H2', 'H2O', 'O2']
    '''
    def __init__(self, file_name):
        '''
        INPUT
        =====
        file_name: string, required
                   name of xml input file
        '''
        self.file_name = file_name
        self.raw = ET.parse(self.file_name).getroot()
        self.species = self.get_species()
        self.reactions = self.get_reactions()
        self.nu_react, self.nu_prod = self.get_nu()
        self.rate_coeff_params = self.get_rate_coeff_params()
        print("Finished reading xml input file")

    def get_species(self):
        '''
        Returns species array from xml file
        '''
        try:
            species = self.raw.find('phase').find('speciesArray').text.strip().split()
        except AttributeError:
            print('ERROR: Check that valid species array is provided in xml input file')
            raise
        if len(species)==0:
            raise ValueError('ERROR: Species array needs to be provided')
        else:
            return species


    def get_reactions(self):
        '''
        Returns list of dictionaries of information about each reaction in the xml file
        '''
        def parse_rate_coeff(reaction, reaction_dict):
            rc_ = reaction.findall('rateCoeff')
            if len(rc_) > 1:
                raise ValueError('ERROR: there are multiple `rateCoeff`s under a reaction')
            elif len(rc_)==0:
                raise ValueError('Rate coefficient data appear to be missing')
            rc_ = rc_[0]
            reaction_dict['rateCoeffParams'] = dict()
            if rc_.find('Constant') is not None:
                reaction_dict['rateCoeffParams']['type'] = 'Constant'
                try:
                    k = float(rc_.find('Constant').find('k').text)

                except AttributeError:
                    print('ERROR: Value of k must be provided for constant rate coefficient')
                    raise
                reaction_dict['rateCoeffParams']['k'] = k

            elif rc_.find('Arrhenius') is not None:
                reaction_dict['rateCoeffParams']['type'] = 'Arrhenius'
                try:
                    A = float(rc_.find('Arrhenius').find('A').text)
                    E = float(rc_.find('Arrhenius').find('E').text)
                except AttributeError:
                    print('ERROR: Values of A and E must be provided for Arrhenius rate coefficient')
                    raise
                reaction_dict['rateCoeffParams']['A'] = A
                reaction_dict['rateCoeffParams']['E'] = E

            elif rc_.find('modifiedArrhenius') is not None:
                reaction_dict['rateCoeffParams']['type'] = 'modifiedArrhenius'
                try:
                    A= float(rc_.find('modifiedArrhenius').find('A').text)
                    b = float(rc_.find('modifiedArrhenius').find('b').text)
                    E = float(rc_.find('modifiedArrhenius').find('E').text)
                except AttributeError:
                    print('ERROR: Values of A, b, and E must be provided for modified Arrhenius rate coefficient')
                    raise
                reaction_dict['rateCoeffParams']['A'] = A
                reaction_dict['rateCoeffParams']['b'] = b
                reaction_dict['rateCoeffParams']['E'] = E
            else:
                raise NotImplementedError('This type of reaction rate coefficient has not been implemented. Current supported types are constant, Arrhenius, and modified Arrhenius.')

        reactions = []
        #check that only one system of reactions is present in the file
        rxndata = self.raw.findall('reactionData')
        if len(rxndata) > 1:
            raise ValueError('ERROR: Only one reaction system allowed in input file')
        elif len(rxndata)==0 or len(rxndata[0])==0:
            raise ValueError('Error: No reactions appear to be present in input file')

        else:
            for i, reaction in enumerate(rxndata[0]):
                reactions.append(deepcopy(reaction.attrib))
                parse_rate_coeff(reaction, reactions[i])
                try:
                    reactions[i]['equation'] = reaction.find('equation').text
                    reactions[i]['reactants'] = {s.split(':')[0]:int(s.split(':')[1]) \
                                         for s in reaction.find('reactants').text.split()}
                    reactions[i]['products'] = {s.split(':')[0]:int(s.split(':')[1]) \
                                         for s in reaction.find('products').text.split()}

                except AttributeError:
                    print("Check that equation, reactants, and products information are present for every reaction")
                    raise

                except ValueError:
                    print("Stoichiometric coefficients must be integers")
                    raise
        return reactions

    def get_nu(self):
        '''
        Return tuple of arrays corresponding to stoichiometric coefficients for reactants and for products
        '''

        nu_react = np.zeros((len(self.species), len(self.reactions)), dtype = int)
        nu_prod = np.zeros((len(self.species), len(self.reactions)), dtype = int)

        for i, reaction in enumerate(self.reactions):
            for specie, stoi in reaction['reactants'].items():
                nu_react[self.species.index(specie), i] = stoi
            for specie, stoi in reaction['products'].items():
                nu_prod[self.species.index(specie), i] = stoi

        return nu_react, nu_prod

    def get_rate_coeff_params(self):
        """getter for rate coefficients"""
        return [reaction['rateCoeffParams'] for reaction in self.reactions]

    def __repr__(self):
        """Return a printable representation of the object."""
        return 'InputParser(file_name=\'{}\')'.format(self.file_name)

    def __len__(self):
        """Return the number of chemical reactions."""
        return len(self.reactions)



import sys
import numpy as np
from pychemkin import InputParser, SQLParser, BackwardCoeffs, ReactionCoeffs

def test_CPcalc_single():
    """
    Test that single reaction case is handled correctly
    """
    ip = InputParser('tests/test_xml/rxns_singlereversible.xml')
    sql = SQLParser()
    bc = BackwardCoeffs(ip.nu_react, ip.nu_prod, ip.species)
    Cp_r = bc.Cp_over_R(500.)
    assert np.all(np.isclose(Cp_r,np.array([ 2.5,3.73848592,2.55540662,3.54595678])))

def test_Hcalc_single():
    """
    Test that single reaction case is handled correctly
    """
    ip = InputParser('tests/test_xml/rxns_singlereversible.xml')
    sql = SQLParser()
    bc = BackwardCoeffs(ip.nu_react, ip.nu_prod, ip.species)
    H_ = bc.H_over_RT(500.)
    assert np.all(np.isclose(H_,np.array([ 53.4473198,1.46400033,60.98145063, 10.90244139])))

def test_coeffs_single():
    """
    Test that single reaction case is handled correctly
    """
    ip = InputParser('tests/test_xml/rxns_singlereversible.xml')
    sql = SQLParser()
    bc = BackwardCoeffs(ip.nu_react, ip.nu_prod, ip.species)
    rc_list = [ReactionCoeffs(**params) for params in ip.rate_coeff_params]
    for rc in rc_list:
        rc.set_params(T = 500)
    kf = np.array([rc.k_forward() for rc in rc_list])
    kb = bc.backward_coeffs(kf, 500)
    assert np.isclose(kb[0],6.03552730e+18)


def test_coeffs_None():
    """
    Check that coefficients are being set correctly
    """
    ip = InputParser('tests/test_xml/rxns_singlereversible.xml')
    sql = SQLParser()
    bc = BackwardCoeffs(ip.nu_react, ip.nu_prod, ip.species)
    bc.coeffs = None
    H_ = bc.H_over_RT(500)
    assert np.all(np.isclose(H_,np.array([ 53.4473198,1.46400033,60.98145063, 10.90244139])))
    bc.coeffs = None
    S_ = bc.S_over_R(500)
    assert np.all(np.isclose(S_, np.array([ 15.08983739,26.54400129,20.70986963,23.94106991])))

def test_no_reversible():
    """
    Check that the empty case can be handled
    """
    ip = InputParser('tests/test_xml/rxns_singlereversible.xml')
    sql = SQLParser()
    bc = BackwardCoeffs(ip.nu_react, ip.nu_prod, ip.species)
    assert len(bc.backward_coeffs(np.array([]), 500))==0

def test_dimension():
    """
    Check that incompatible input array sizes are correctly caught
    """
    ip = InputParser('tests/test_xml/rxns_singlereversible.xml')
    sql = SQLParser()
    bc = BackwardCoeffs(ip.nu_react, ip.nu_prod, ip.species)
    try:
        bc.backward_coeffs(np.array([1, 2, 3]), 500)
    except ValueError as e:
        assert type(e) == ValueError
        print(e)



"""setup file

require libraries like numpy...

Some setup code borrowed from https://github.com/activescott/python-package-example
"""

from setuptools import setup, find_packages
from codecs import open
from os import path

here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(here, 'README.rst'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='pychemkin',

    version='1.0.8',

    description='Chemical Kinetics Library',
    long_description=long_description,

    # The project's main homepage.
    url='https://github.com/cs207group4/cs207-FinalProject',

    # Author details
    author='CS207 Group4',
    author_email = 'zzy8200@gmail.com',

    # Choose your license
    license='GNU General Public License v3 (GPLv3)',

    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Science/Research',
        'Topic :: Scientific/Engineering :: Chemistry',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],

    keywords= 'chemical kinetics',

    packages= ['pychemkin'],

    install_requires=['numpy','scipy','pandas'],

    # List additional groups of dependencies here (e.g. development
    # dependencies). You can install these using the following syntax,
    # for example:
    # $ pip install -e .[dev,test]
    setup_requires = ['pytest-runner','numpy'],
    tests_require = ['pytest','coverage','coveralls','pytest-cov'],
    test_suite='tests',
    include_package_data=True
)



'''

pychemkin
~~~~~~~~~

A python library for computing chemical kinetics. For details, please refer to the user manual. The main class user should use is chemkin.

:copyright: (c) 2017 by cs207 group4
:license: GPLv3, see LICENSE.txt for details

'''

from .InputParser import InputParser
from .SQLParser import SQLParser
from .BackwardCoeffs import BackwardCoeffs
from .ReactionCoeffs import ReactionCoeffs
from .chemkin import chemkin
